import {
  Avatar,
  Button,
  Collapse,
  Input,
  Popconfirm,
  Space,
  Tag,
  Tooltip,
  Tree,
  notification,
} from "antd";
import {
  Constants,
  MyAvatar,
  SentMessagesCommands,
  WebSocketContext,
  hasPermission,
} from "../../../utils";
import {
  ArrowDownOutlined,
  ArrowUpOutlined,
  CloseOutlined,
  DeleteOutlined,
  EditOutlined,
  PlusOutlined,
  QuestionCircleOutlined,
  SaveOutlined,
} from "@ant-design/icons";
import { useContext, useState } from "react";
import { useTranslation } from "react-i18next";

export default function AdminAreaRoles() {
  const { t } = useTranslation();
  const webSocketContext = useContext(WebSocketContext);
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();

  function createTreeDataList(permissions) {
    const result = [];

    for (let i = 0; i < permissions.length; i++) {
      const argument = permissions[i];
      const keys = argument.split(".");
      let currentParent = null;

      for (let j = 0; j < keys.length; j++) {
        const key = keys[j];
        const title = key
          .replace(new RegExp("_", "g"), " ")
          .replace(/^\w/, (c) => c.toUpperCase());
        const obj = {
          title: title,
          key: keys.slice(0, j + 1).join("."),
          children: [],
        };

        if (j === 0) {
          const existingObj = result.find((item) => item.title === obj.title);
          if (existingObj) {
            currentParent = existingObj;
          } else {
            result.push(obj);
            currentParent = obj;
          }
        } else {
          const existingChild = currentParent.children.find(
            (item) => item.title === obj.title
          );
          if (existingChild) {
            currentParent = existingChild;
          } else {
            const childObj = {
              title: title,
              key: keys.slice(0, j + 1).join("."),
              children: [],
            };
            currentParent.children.push(childObj);
            currentParent = childObj;
          }
        }
      }
    }

    return result;
  }

  const onCreateNewRoleClick = () => {
    webSocketContext.SendSocketMessage(
      SentMessagesCommands.AdminAreaCreateNewRole,
      {}
    );
  };

  const getRolePermissions = (role) => {
    const rolePermissions = webSocketContext.AdminAreaRolesPermissions.find(
      (r) => r.RoleId === role.Id
    );

    if (rolePermissions === undefined || rolePermissions.Permissions === null) {
      return [];
    }

    return rolePermissions.Permissions;
  };

  const treeData = createTreeDataList(
    getRolePermissions(
      webSocketContext.AllRoles.find((role) => role.Master === true)
    )
  );

  return (
    <Space direction="vertical" style={{ width: "100%" }}>
      {notificationContextHolder}
      {webSocketContext.AllRoles.map((role) => (
        <Role
          key={role.Id}
          treeData={treeData}
          role={role}
          webSocketContext={webSocketContext}
          notificationApi={notificationApi}
        />
      ))}

      {hasPermission(
        webSocketContext.User.Permissions,
        Constants.PERMISSIONS.ADMIN_AREA.ROLES.CREATE_NEW_ROLE
      ) && (
        <div
          style={{
            display: "flex",
            justifyContent: "center",
          }}
        >
          <Popconfirm
            placement="top"
            okText={t("adminArea.createNewRole.popconfirm.okText")}
            cancelText={t("buttonCancel")}
            title={t("adminArea.createNewRole.popconfirm.title")}
            onConfirm={() => onCreateNewRoleClick()}
          >
            <Button shape="round" icon={<PlusOutlined />} size="large">
              {t("adminArea.createNewRole.button")}
            </Button>
          </Popconfirm>
        </div>
      )}
    </Space>
  );
}

function Role({ treeData, role, webSocketContext, notificationApi }) {
  const { t } = useTranslation();
  const [editMode, setEditMode] = useState(false);
  const [roleDisplayName, setRoleDisplayName] = useState("");
  const [roleDescription, setRoleDescription] = useState("");

  const getRolePermissions = (role) => {
    const rolePermissions = webSocketContext.AdminAreaRolesPermissions.find(
      (r) => r.RoleId === role.Id
    );

    if (rolePermissions === undefined || rolePermissions.Permissions === null) {
      return [];
    }

    return rolePermissions.Permissions;
  };

  const rolePermissions = getRolePermissions(role);

  const [checkedTreeKeyPermissions, setCheckedTreeKeyPermissions] =
    useState(rolePermissions);

  const onTreeCheck = (checkedKeys) =>
    setCheckedTreeKeyPermissions(checkedKeys);

  const getMasterPermissions = () => {
    return webSocketContext.AdminAreaRolesPermissions.find(
      (role) =>
        role.RoleId ===
        webSocketContext.AllRoles.find((role) => role.Master === true).Id
    ).Permissions;
  };

  const getTreePermissionsChanges = () => {
    const masterPermissions = getMasterPermissions();

    // the checkedKeys permissions must be filtered out, because permissions like admin_area.roles are specified in keys, but are not a permission, but only the children of it admin_area.roles.add_role
    const potentialPermissions = checkedTreeKeyPermissions.filter(
      (permission) =>
        masterPermissions.find((mPermission) => mPermission === permission)
    );

    const addedPermissions = potentialPermissions.filter(
      (permission) => !rolePermissions.includes(permission)
    );

    const removedPermissions = rolePermissions.filter(
      (permission) => !potentialPermissions.includes(permission)
    );

    return {
      addedPermissions,
      removedPermissions,
    };
  };

  const onEditClick = () => {
    setCheckedTreeKeyPermissions(rolePermissions);
    setEditMode(true);

    setRoleDisplayName(role.DisplayName);
    setRoleDescription(role.Description);
  };

  const onSaveClick = () => {
    let changes = {};

    const permissionChanges = getTreePermissionsChanges();

    if (roleDisplayName !== role.DisplayName) {
      changes["DisplayName"] = roleDisplayName;
    }
    if (roleDescription !== role.Description) {
      changes["Description"] = roleDescription;
    }
    if (permissionChanges.addedPermissions.length > 0) {
      changes["AddedPermissions"] = permissionChanges.addedPermissions;
    }
    if (permissionChanges.removedPermissions.length > 0) {
      changes["RemovedPermissions"] = permissionChanges.removedPermissions;
    }

    // user has clicked on save icon but has nothing changed
    if (
      changes.DisplayName === undefined &&
      changes.Description === undefined &&
      permissionChanges.addedPermissions.length === 0 &&
      permissionChanges.removedPermissions.length === 0
    ) {
      setEditMode(false);
      return;
    }

    if (changes.DisplayName !== undefined) {
      if (
        changes.DisplayName.length < Constants.GLOBALS.MIN_ROLE_DISPLAY_NAME
      ) {
        notificationApi["error"]({
          message: t(
            "adminArea.roleUpdate.errorDisplayNameToShort.notification.message",
            { MIN_ROLE_DISPLAY_NAME: Constants.GLOBALS.MIN_ROLE_DISPLAY_NAME }
          ),
          description: t(
            "adminArea.roleUpdate.errorDisplayNameToShort.notification.description"
          ),
        });
        return;
      }
      if (
        changes.DisplayName.length > Constants.GLOBALS.MAX_ROLE_DISPLAY_NAME
      ) {
        notificationApi["error"]({
          message: t(
            "adminArea.roleUpdate.errorDisplayNameToLong.notification.message",
            { MAX_ROLE_DISPLAY_NAME: Constants.GLOBALS.MAX_ROLE_DISPLAY_NAME }
          ),
          description: t(
            "adminArea.roleUpdate.errorDisplayNameToLong.notification.description"
          ),
        });
        return;
      }
    }

    if (
      changes.Description !== undefined &&
      changes.Description.length > Constants.GLOBALS.MAX_ROLE_DESCRIPTION
    ) {
      notificationApi["error"]({
        message: t(
          "adminArea.roleUpdate.errorDescriptionToLong.notification.message",
          { MAX_ROLE_DESCRIPTION: Constants.GLOBALS.MAX_ROLE_DESCRIPTION }
        ),
        description: t(
          "adminArea.roleUpdate.errorDescriptionToLong.notification.description"
        ),
      });
      return;
    }

    setEditMode(false);

    webSocketContext.SendSocketMessage(
      SentMessagesCommands.AdminAreaUpdateRole,
      {
        RoleId: role.Id,
        Changes: changes,
      }
    );
  };

  const onCloseClick = () => {
    setEditMode(false);

    setRoleDisplayName(role.DisplayName);
    setRoleDescription(role.Description);
  };

  const onMoveUpClick = () => {
    webSocketContext.SendSocketMessage(
      SentMessagesCommands.AdminAreaUpdateRoleSortingOrder,
      {
        RoleId: role.Id,
        Direction: 0,
      }
    );
  };

  const onMoveDownClick = () => {
    webSocketContext.SendSocketMessage(
      SentMessagesCommands.AdminAreaUpdateRoleSortingOrder,
      {
        RoleId: role.Id,
        Direction: 1,
      }
    );
  };

  const getUsersInRole = () => {
    return webSocketContext.AllUsers.filter((user) => user.RoleId === role.Id);
  };

  const UserAvatarsInRole = () => {
    const usersInRole = getUsersInRole();

    if (usersInRole.length > 0) {
      let avatars = [];

      usersInRole.forEach((user) => {
        avatars.push(
          <MyAvatar
            key={user.Id}
            avatarWidth={26}
            avatar={user.Avatar}
            tooltip
            tooltipTitle={user.Username}
          />
        );
      });

      return (
        <Avatar.Group
          maxCount={6}
          size={28}
          maxStyle={{ color: "#f56a00", backgroundColor: "#fde3cf" }}
        >
          {avatars}
        </Avatar.Group>
      );
    }

    return null;
  };

  const onDeleteClick = () => {
    webSocketContext.SendSocketMessage(
      SentMessagesCommands.AdminAreaDeleteRole,
      { RoleId: role.Id }
    );
  };

  const getMaxRoleSortingOrder = () => {
    return webSocketContext.AllRoles[webSocketContext.AllRoles.length - 1]
      .SortingOrder;
  };

  return (
    <Collapse
      collapsible={editMode ? "icon" : "header"}
      defaultActiveKey={["2"]}
      items={[
        {
          key: "2",
          extra: editMode ? (
            <Space key="spaceedit" style={{ paddingLeft: 10 }}>
              {hasPermission(
                webSocketContext.User.Permissions,
                Constants.PERMISSIONS.ADMIN_AREA.ROLES.DELETE_ROLE
              ) && (
                <>
                  {role.Master || getUsersInRole().length > 0 ? (
                    <DeleteOutlined
                      style={{
                        color: "rgba(0, 0, 0, 0.25)",
                        cursor: "not-allowed",
                      }}
                    />
                  ) : (
                    <Popconfirm
                      title={t("adminArea.deleteRole.popconfirm.title")}
                      description={t(
                        "adminArea.deleteRole.popconfirm.description"
                      )}
                      icon={<QuestionCircleOutlined style={{ color: "red" }} />}
                      cancelText={t("buttonCancel")}
                      onConfirm={() => onDeleteClick()}
                    >
                      <Tooltip title={t("adminArea.deleteRole.tooltip.title")}>
                        <DeleteOutlined />
                      </Tooltip>
                    </Popconfirm>
                  )}
                </>
              )}

              {hasPermission(
                webSocketContext.User.Permissions,
                Constants.PERMISSIONS.ADMIN_AREA.ROLES.MOVE_ROLE_UP_DOWN
              ) && (
                <>
                  {role.SortingOrder === 0 ? (
                    <ArrowUpOutlined
                      style={{
                        color: "rgba(0, 0, 0, 0.25)",
                        cursor: "not-allowed",
                      }}
                    />
                  ) : (
                    <Tooltip title={t("adminArea.moveRoleUp.tooltip.title")}>
                      <ArrowUpOutlined onClick={() => onMoveUpClick()} />
                    </Tooltip>
                  )}
                  {role.SortingOrder === getMaxRoleSortingOrder() ? (
                    <ArrowDownOutlined
                      style={{
                        color: "rgba(0, 0, 0, 0.25)",
                        cursor: "not-allowed",
                      }}
                    />
                  ) : (
                    <Tooltip title={t("adminArea.moveRoleDown.tooltip.title")}>
                      <ArrowDownOutlined onClick={() => onMoveDownClick()} />
                    </Tooltip>
                  )}
                </>
              )}

              {hasPermission(
                webSocketContext.User.Permissions,
                Constants.PERMISSIONS.ADMIN_AREA.ROLES.UPDATE_ROLE
              ) && (
                <Tooltip title={t("adminArea.save.tooltip.title")}>
                  <SaveOutlined onClick={() => onSaveClick()} />
                </Tooltip>
              )}

              <Tooltip title={t("adminArea.close.tooltip.title")}>
                <CloseOutlined onClick={() => onCloseClick()} />
              </Tooltip>
            </Space>
          ) : (
            <Space style={{ paddingLeft: 10 }} align="start">
              <UserAvatarsInRole />

              <Tooltip title={t("adminArea.edit.tooltip.title")}>
                <EditOutlined onClick={() => onEditClick()} />
              </Tooltip>
            </Space>
          ),
          label: (
            <>
              {editMode ? (
                <>
                  <Input
                    style={{
                      fontWeight: "bold",
                      color: Constants.COLORS.PRIMARY,
                    }}
                    value={roleDisplayName}
                    onChange={(e) => setRoleDisplayName(e.target.value)}
                    minLength={Constants.GLOBALS.MIN_ROLE_DISPLAY_NAME}
                    maxLength={Constants.GLOBALS.MAX_ROLE_DISPLAY_NAME}
                  />

                  <Input
                    value={roleDescription}
                    onChange={(e) => setRoleDescription(e.target.value)}
                    maxLength={Constants.GLOBALS.MAX_ROLE_DESCRIPTION}
                  />
                </>
              ) : (
                <>
                  <h1
                    style={{
                      margin: 0,
                      fontWeight: "bold",
                      color: Constants.COLORS.PRIMARY,
                    }}
                  >
                    {role.DisplayName}{" "}
                    {role.Master && <Tag color="purple">Master</Tag>}
                  </h1>

                  <span style={{ margin: 0 }}>{role.Description}</span>
                </>
              )}
            </>
          ),
          children: (
            <>
              {role.Master && editMode && (
                <p style={{ fontStyle: "italic" }}>
                  {t("adminArea.masterRoleRightsMessage")}
                </p>
              )}

              {editMode ? (
                <Tree
                  checkable
                  disabled={role.Master ? true : editMode ? false : true}
                  selectable={false}
                  checkedKeys={checkedTreeKeyPermissions}
                  onCheck={onTreeCheck}
                  treeData={treeData}
                />
              ) : (
                <Tree
                  checkable
                  disabled={true}
                  selectable={false}
                  checkedKeys={rolePermissions}
                  treeData={treeData}
                />
              )}
            </>
          ),
        },
      ]}
    />
  );
}
